using System;
using Antlr.Runtime;
using Antlr.Runtime.Tree;
using Microsoft.Scripting;
using Microsoft.Scripting.Ast;
using Microsoft.Scripting.Runtime;
using Microsoft.Scripting.Shell;
using MyL.Parser.AST;
using MyL.Runtime;

namespace MyL {
	///<summary>
	///Provides language specific facilities which are typically called by the runtime.
	///</summary> 
	class MyLLanguageContext : LanguageContext {

		public MyLLanguageContext(ScriptDomainManager manager)
			: base(manager) {
			Binder = new MyLBinder(new CodeContext(new Scope(this), this));
		}

		public override LambdaExpression ParseSourceCode(CompilerContext context) {
			LambdaBuilder lb;
			CommonTokenStream tokenStream;
			CommonTree t;
			TestLexerLexer lexer;
			TestParserParser parser;
			CommonTreeNodeStream nodes;
			TestTree walker;
			MyLStatement s;

			s = null;
			try {
				lexer = new TestLexerLexer(new ANTLRStringStream(context.SourceUnit.GetCode()));
				tokenStream = new CommonTokenStream(lexer);
				parser = new TestParserParser(tokenStream);
				TestParserParser.prog_return r = parser.prog();
				t = (CommonTree)r.Tree;
				nodes = new CommonTreeNodeStream(t);
				walker = new TestTree(nodes);
				s = walker.prog();
			} catch (Exception ex) {
				Console.WriteLine(ex.Message);
			}

            lb = Ast.Lambda("<mylblock>", typeof(object));
            lb.Body = s.Generate();

			return lb.MakeLambda();
		}

		public override ServiceType GetService<ServiceType>(params object[] args) {
			if (typeof(ServiceType) == typeof(OptionsParser)) {
				return (ServiceType)(object)new DefaultOptionsParser(this);
			} else if (typeof(ServiceType) == typeof(CommandLine)) {
				return (ServiceType)(object)new CommandLine();
			}

			return base.GetService<ServiceType>(args);
		}
	}
}
